// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "dlgPreferences.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2004
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

unit dlgPreferences;

interface

uses
  uVersionCheck,
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, ActnList, Dialogs, Mask;

type
  TUserPreferences = record
    CheckForUpdate: Boolean;
    HttpProxy: string;
    HttpProxyPassword: string;
    HttpProxyPort: Integer;
    HttpProxyUsername: string;
    StoreHttpProxyPassword: Integer;
    UseProxies: Boolean;
  end;

  TVersionCheckerCompleted = procedure(Sender: TObject;
                                       VCResult: TVersionCheckerResult) of object;

  TPreferencesDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    MainPageControl: TPageControl;
    ProxiesTabSheet: TTabSheet;
    ProxyConfigurationGroupBox: TGroupBox;
    DirectConnectionBtn: TRadioButton;
    ManualProxyBtn: TRadioButton;
    HttpProxyEdit: TLabeledEdit;
    HttpProxyPortEdit: TLabeledEdit;
    ForgetProxyPasswordBtn: TButton;
    ProxyAuthenticationGroupBox: TGroupBox;
    Label1: TLabel;
    HttpRememberPasswordPermanentlyBtn: TRadioButton;
    HttpRememberPasswordTemporarilyBtn: TRadioButton;
    Label2: TLabel;
    HttpDoNotRememberPasswordBtn: TRadioButton;
    Label3: TLabel;
    PasswordsTabSheet: TTabSheet;
    UpdateNotificationTabSheet: TTabSheet;
    UpdateNotificationGroupBox: TGroupBox;
    CheckVersionOnBtn: TRadioButton;
    CheckVersionOffBtn: TRadioButton;
    CheckNowBtn: TButton;
    Label4: TLabel;
    Label5: TLabel;
    ToolsTabSheet: TTabSheet;
    HexEditorGroupBox: TGroupBox;
    HexEditorPathEdit: TLabeledEdit;
    HexEditorPathButton: TButton;
    ToolsOpenDialog: TOpenDialog;
    Label6: TLabel;
    Label7: TLabel;
    SaxonGroupBox: TGroupBox;
    Label8: TLabel;
    SaxonPathEdit: TLabeledEdit;
    SaxonPathButton: TButton;
    FontTabSheet: TTabSheet;
    FontGroupBox: TGroupBox;
    lblFontName: TLabel;
    lblFontSize: TLabel;
    FontPreviewPanel: TPanel;
    FontNameListBox: TListBox;
    FontSizeListBox: TListBox;
    FontSizeEdit: TEdit;
    procedure ForgetProxyPasswordBtnClick(Sender: TObject);
    procedure SwitchProxyConfiguration(Sender: TObject);
    procedure CheckForNewVersion(Sender: TObject);
    procedure HexEditorPathButtonClick(Sender: TObject);
    procedure SaxonPathButtonClick(Sender: TObject);
    procedure FontSizeListBoxClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FontNameListBoxClick(Sender: TObject);
    procedure FontSizeEditChange(Sender: TObject);
    procedure FontSizeEditKeyPress(Sender: TObject; var Key: Char);
  private
    FCheckForUpdateAction: TAction;
    FOnForgetPwd: TNotifyEvent;
    function GetCheckForUpdate: Boolean;
    function GetEditorFont: TFont;
    function GetHexEditor: string;
    function GetSaxon: string;
    function GetHttpProxy: string;
    function GetHttpProxyPort: Integer;
    function GetStoreHttpProxyPassword: Integer;
    function GetUseProxies: Boolean;
    procedure SetCheckForUpdate(const Value: Boolean);
    procedure SetEditorFont(const Value: TFont);
    procedure SetHttpProxy(const Value: string);
    procedure SetHttpProxyPort(const Value: Integer);
    procedure SetStoreHttpProxyPassword(const Value: Integer);
    procedure SetUseProxies(const Value: Boolean);
    procedure SetHexEditor(const Value: string);
    procedure SetSaxon(const Value: string);
    procedure UpdateFontSizeList(const ListBox: TCustomListBox; const FontName: TFontName);
    function GetPixelsPerInch: Integer;

    property PixelsPerInch: Integer read GetPixelsPerInch;
  public
    property CheckForUpdate: Boolean read GetCheckForUpdate write SetCheckForUpdate;
    property CheckForUpdateAction: TAction read FCheckForUpdateAction write FCheckForUpdateAction;
    property EditorFont: TFont read GetEditorFont write SetEditorFont;
    property HexEditor: string read GetHexEditor write SetHexEditor;
    property HttpProxy: string read GetHttpProxy write SetHttpProxy;
    property HttpProxyPort: Integer read GetHttpProxyPort write SetHttpProxyPort;
    property Saxon: string read GetSaxon write SetSaxon;
    property StoreHttpProxyPassword: Integer read GetStoreHttpProxyPassword write SetStoreHttpProxyPassword;
    property UseProxies: Boolean read GetUseProxies write SetUseProxies;

    property OnForgetPwd: TNotifyEvent read FOnForgetPwd write FOnForgetPwd;
  end;

var
  UserPreferences: TUserPreferences;

implementation

{$R *.dfm}

uses
  AdomNetwork_5_1;

procedure TPreferencesDlg.FontNameListBoxClick(Sender: TObject);
var
  Size: Integer;
begin
  if FontNameListBox.ItemIndex <> -1 then begin
    FontPreviewPanel.Font.Name := FontNameListBox.Items[FontNameListBox.ItemIndex];
    UpdateFontSizeList(FontSizeListBox, FontPreviewPanel.Font.Name);
  end;
  Size := FontPreviewPanel.Font.Size;
  FontSizeListBox.ItemIndex := FontSizeListBox.Items.IndexOf(IntToStr(Size));
  FontSizeEdit.Text := IntToStr(Size);
end;

procedure TPreferencesDlg.FontSizeEditChange(Sender: TObject);
var
  Size: Integer;
begin
  Size := StrToIntDef(FontSizeEdit.Text, FontPreviewPanel.Font.Size);
  FontPreviewPanel.Font.Size := Size;
  FontSizeListBox.ItemIndex := FontSizeListBox.Items.IndexOf(IntToStr(Size));
end;

procedure TPreferencesDlg.FontSizeEditKeyPress(Sender: TObject; var Key: Char);
begin
  // Allow only numbers, BACKSPACE and DELETE.
  case Key of
      '0'..'9', #$08, #$2E:;
    else
      Key := #0;
  end;
end;

procedure TPreferencesDlg.FontSizeListBoxClick(Sender: TObject);
begin
  FontSizeEdit.Text := IntToStr(StrToIntDef(FontSizeListBox.Items[FontSizeListBox.ItemIndex], FontPreviewPanel.Font.Size));
end;

procedure TPreferencesDlg.ForgetProxyPasswordBtnClick(Sender: TObject);
begin
  if Assigned(FOnForgetPwd) then FOnForgetPwd(Self);
  ForgetProxyPasswordBtn.Enabled := False;
end;

procedure TPreferencesDlg.FormCreate(Sender: TObject);
begin
  FontNameListBox.Items := Screen.Fonts;
end;

function TPreferencesDlg.GetCheckForUpdate: Boolean;
begin
  Result := CheckVersionOnBtn.Checked;
end;

function TPreferencesDlg.GetEditorFont: TFont;
begin
  Result := FontPreviewPanel.Font;
end;

function TPreferencesDlg.GetHexEditor: string;
begin
  Result := HexEditorPathEdit.Text;
end;

function TPreferencesDlg.GetSaxon: string;
begin
  Result := SaxonPathEdit.Text;
end;

function TPreferencesDlg.GetHttpProxy: string;
begin
  Result := HttpProxyEdit.Text;
end;

function TPreferencesDlg.GetHttpProxyPort: Integer;
begin
  if not TryStrToInt(HttpProxyPortEdit.Text, Result) then
    Result := 0;
end;

function TPreferencesDlg.GetPixelsPerInch: Integer;
var
  DC: HDC;
begin
  DC := GetDC(0);
  try
    Result := GetDeviceCaps(DC, LOGPIXELSY);
  finally
    ReleaseDC(0, DC);
  end;
end;

function TPreferencesDlg.GetStoreHttpProxyPassword: Integer;
begin
  if HttpRememberPasswordPermanentlyBtn.Checked then begin
    Result := 0;
  end else if HttpRememberPasswordTemporarilyBtn.Checked then begin
    Result := 1;
  end else
    Result := 2;
end;

procedure TPreferencesDlg.SetCheckForUpdate(const Value: Boolean);
begin
  CheckVersionOnBtn.Checked := Value;
  CheckVersionOffBtn.Checked := not Value;
end;

procedure TPreferencesDlg.SetEditorFont(const Value: TFont);
var
  Index: Integer;
begin
  FontPreviewPanel.Font := Value;
  Index := FontNameListBox.Items.IndexOf(Value.Name);
  if (Index <> -1) and (Index <> FontNameListBox.ItemIndex) then begin
    FontNameListBox.ItemIndex := Index;
    FontNameListBox.OnClick(FontNameListBox);
  end;
end;

procedure TPreferencesDlg.SetHexEditor(const Value: string);
begin
  HexEditorPathEdit.Text := Value;
end;

procedure TPreferencesDlg.SetSaxon(const Value: string);
begin
  SaxonPathEdit.Text := Value;
end;

procedure TPreferencesDlg.SetHttpProxy(const Value: string);
begin
  HttpProxyEdit.Text := Value;
  SwitchProxyConfiguration(Self);
end;

procedure TPreferencesDlg.SetHttpProxyPort(const Value: Integer);
begin
  if Value < 1
    then HttpProxyPortEdit.Text := ''
    else HttpProxyPortEdit.Text := IntToStr(Value);
end;

procedure TPreferencesDlg.SetStoreHttpProxyPassword(const Value: Integer);
begin
  Case Value of
    0: HttpRememberPasswordPermanentlyBtn.Checked := True;
    1: HttpRememberPasswordTemporarilyBtn.Checked := True;
  else
    HttpDoNotRememberPasswordBtn.Checked := True;
  end;
end;

procedure TPreferencesDlg.SetUseProxies(const Value: Boolean);
begin
  ManualProxyBtn.Checked := Value;
  SwitchProxyConfiguration(Self);
end;

procedure TPreferencesDlg.SwitchProxyConfiguration(Sender: TObject);
begin
  HttpProxyEdit.Enabled := ManualProxyBtn.Checked;
  HttpProxyPortEdit.Enabled := ManualProxyBtn.Checked;
end;

procedure TPreferencesDlg.UpdateFontSizeList(const ListBox: TCustomListBox;
  const FontName: TFontName);
var
  DC: HDC;

  function EnumFontSizes(var EnumLogFont: TEnumLogFontEx;
    PTextMetric: PNewTextMetric; FontType: Integer; Data: LPARAM): Integer;
    export; stdcall;
  var
    S: String;
    I, Value, Value2: Integer;
  begin
    if (FontType and TRUETYPE_FONTTYPE) <> 0 then begin
      TStrings(Data).Add('8');
      TStrings(Data).Add('9');
      TStrings(Data).Add('10');
      TStrings(Data).Add('11');
      TStrings(Data).Add('12');
      TStrings(Data).Add('14');
      TStrings(Data).Add('16');
      TStrings(Data).Add('18');
      TStrings(Data).Add('20');
      TStrings(Data).Add('22');
      TStrings(Data).Add('24');
      TStrings(Data).Add('26');
      TStrings(Data).Add('28');
      TStrings(Data).Add('36');
      TStrings(Data).Add('48');
      TStrings(Data).Add('72');
      Result := 0;
      end
    else begin
      Value := Round((EnumLogFont.elfLogFont.lfHeight - PTextMetric.tmInternalLeading) * 72 / PixelsPerInch);
      S := IntToStr(Value);
      Result := 1;
      for I := 0 to Pred(TStrings(Data).Count) do begin
        Value2 := StrToInt(TStrings(Data)[I]);
        if Value2 = Value then
          Exit;
        if Value2 > Value then begin
          TStrings(Data).Insert(I,S);
          Exit;
        end;
      end;
      TStrings(Data).Add(S);
    end;
  end;

begin
  with ListBox do begin
    Items.BeginUpdate;
    try
      Items.Clear;
      if FontName <> '' then begin
        DC := GetDC(0);
        try
          EnumFontFamilies(DC, PWideChar(FontName), @EnumFontSizes, Integer(Items));
        finally
          ReleaseDC(0, DC);
        end;
      end;
    finally
      Items.EndUpdate;
    end;
  end;
end;

function TPreferencesDlg.GetUseProxies: Boolean;
begin
  Result := ManualProxyBtn.Checked;
end;

procedure TPreferencesDlg.HexEditorPathButtonClick(Sender: TObject);
begin
  if ToolsOpenDialog.Execute then
    HexEditorPathEdit.Text := ToolsOpenDialog.FileName;
end;

procedure TPreferencesDlg.SaxonPathButtonClick(Sender: TObject);
begin
  if ToolsOpenDialog.Execute then
    SaxonPathEdit.Text := ToolsOpenDialog.FileName;
end;

procedure TPreferencesDlg.CheckForNewVersion(Sender: TObject);
begin
  if Assigned(FCheckForUpdateAction) then
    FCheckForUpdateAction.Execute;
end;

end.
