// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "frameXmlEdit.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2004
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.

unit frameXmlEdit;

interface

uses
  uXmlEditTypes, uXmlDocMan, dkUriUtils, dkAdomCore, AdomErrorReporting_5_1,
  SynEditHighlighter, SynEditPrint, SynEditTypes, ActnList,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Buttons, frameXmlWinEdit, ExtCtrls,
  AutoListUtils, ComCtrls, ImgList, ToolWin, ActnMan,
  ActnCtrls, StdCtrls, AppLauncher;

type
  TXmlEditFrame = class(TFrame)
    XmlDocManager: TXmlDocManager;
    RawDataMessagePanel: TPanel;
    Label1: TLabel;
    ConfigureHexEditorPanel: TPanel;
    Label2: TLabel;
    LaunchPreferencesBtn: TButton;
    LaunchHexEditorPanel: TPanel;
    LaunchHexEditorBtn: TButton;
    XmlWinEdit: TXmlWinEdit;
    procedure LaunchPreferencesBtnClick(Sender: TObject);
    procedure LaunchHexEditorBtnClick(Sender: TObject);
  private
    FExternalHexEditor: TFilename;
    FXmlEditor: IXmlEditCtrl;
    FOnSetupHexEditorRequest: TNotifyEvent;
    function GetEditorFont: TFont;
    function GetErrorAutoStrings: TDomErrorAutoStrings;
    function GetIsWellformed: Boolean;
    function GetIsXml: Boolean;
    function GetResourceResolver: TCustomResourceResolver;
    function GetXmlHighlighter: TSynCustomHighlighter;
    procedure SetEditorFont(const Value: TFont);
    procedure SetExternalHexEditor(const Value: TFilename);
    procedure SetResourceResolver(const Value: TCustomResourceResolver);
    procedure SetTextWin(AText: TBytes;
                         AModified: Boolean);  { TODO 5 -cRefactor : Rename }
    procedure SetXmlHighlighter(const Value: TSynCustomHighlighter);
    procedure UpdateDisplays(const AModified: Boolean);
    procedure UpdateXmlDocManager(const IncDecl: Boolean);
  protected
    procedure DoSetupHexEditorRequest; virtual;
    function GetDocumentUri: WideString; virtual;
    function GetEditorState: string; virtual;
    function GetMimeType: string;
    function GetModified: Boolean; virtual;
    procedure HideEditor; virtual;
    procedure SetDocumentUri(const AUri: WideString); virtual;
    procedure SetModified(const Value: Boolean); virtual;
    procedure ShowEditor; virtual;
  public
    constructor Create(      AOwner: TComponent;
                       const ASourceCode: TBytes;
                       const ADocumentUri: WideString;
                       const AModified: Boolean); reintroduce;
    destructor Destroy; override;
    function CanCheckValidity: Boolean; virtual;
    function CanCheckWellformedness: Boolean; virtual;
    function CanCopy: Boolean; virtual;
    function CanCut: Boolean; virtual;
    function CanDelete: Boolean; virtual;
    function CanPaste: Boolean; virtual;
    function CanPrint: Boolean; virtual;
    function CanRedo: Boolean; virtual;
    function CanReplace: Boolean; virtual;
    function CanSave: Boolean; virtual;
    function CanSaveAs: Boolean; virtual;
    function CanSearch: Boolean; virtual;
    function CanSelectAll: Boolean; virtual;
    function CanUndo: Boolean; virtual;
    function CaretPosStr: string; virtual;
    function CheckValidity: Boolean; virtual;
    function CheckWellformedness: Boolean; virtual;
    procedure CopyToClipboard; virtual;
    procedure CutToClipboard; virtual;
    procedure DeleteSelection; virtual;
    function FocusEditor: Boolean; virtual;
    function HasErrorStrings: Boolean; virtual;
    function IsEmpty: Boolean; virtual;
    procedure LoadFromBytes(const Data: TBytes;
                            const AModified: Boolean); virtual;
    procedure PasteFromClipboard; virtual;
    procedure Print(const ASynEditPrint: TSynEditPrint); virtual;
    procedure PrintPreview(const ASynEditPrint: TSynEditPrint;
                           const PrinterSetupAction: TAction); virtual;
    procedure Redo; virtual;
    function Save: Boolean; virtual;
    procedure SearchReplace(const ASearch, AReplace: WideString;
      AOptions: TSynSearchOptions); virtual;
    procedure SelectAll; virtual;
    function SetCaret(const Location: IDomLocator): Boolean; virtual;
    function StartReplace(var Params: TXmlSearchParams): Boolean; virtual;
    function StartSearch(var Params: TXmlSearchParams): Boolean; virtual;
    procedure Undo; virtual;

    property DocumentUri: WideString read GetDocumentUri write SetDocumentUri;
    property EditorFont: TFont read GetEditorFont write SetEditorFont;
    property EditorState: string read GetEditorState;
    property ErrorAutoStrings: TDomErrorAutoStrings read GetErrorAutoStrings;
    property ExternalHexEditor: TFilename read FExternalHexEditor write SetExternalHexEditor;
    property IsWellformed: Boolean read GetIsWellformed;
    property IsXml: Boolean read GetIsXml;
    property MimeType: string read GetMimeType;
    property Modified: Boolean read GetModified write SetModified;
    property ResourceResolver: TCustomResourceResolver read GetResourceResolver write SetResourceResolver;
    property XmlHighlighter: TSynCustomHighlighter read GetXmlHighlighter write SetXmlHighlighter;

    property OnSetupHexEditorRequest: TNotifyEvent read FOnSetupHexEditorRequest write FOnSetupHexEditorRequest;
  end;

implementation

uses
  dkParserUtilsWin32, AdomTranscoder_5_1, ShellAPI;

{$R *.dfm}

resourcestring
  SCannotDisplay = 'Cannot switch to text mode.'#13#10#13#10 +   { TODO 5 -cRefactor : Remove SCannotDisplay? }
                   'The source code contains markup characters which are not covered by the current system encoding (%s).';

  SCannotConvert = 'Cannot convert the source code into %s.'#13#10#13#10 +
                   'Saving the document "as is" but with line breaks normalized.';

  SUnknownMimeType = 'Internal error: Unknown MIME type "%s".';

  SHexEditorNotFound = 'The external hex-editor could not be found.'#13#10#13#10 +
                       'Do you want to check its path?';

  SHexEditorLaunchFailed = 'An error occurred while attempting to launch the external hex-editor.'#13#10#13#10 +
                           'Do you want to check its path?';

{ Helper functions }

procedure SaveBytes(const B: TBytes; const FileName: TFilename);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
      Stream.WriteBuffer(B[0], Length(B));
  finally
    Stream.Free;
  end;
end;

{ TXmlEditFrame }

function TXmlEditFrame.CanCheckValidity: Boolean;
begin
  Result := IsXml;
end;

function TXmlEditFrame.CanCheckWellformedness: Boolean;
begin
  Result := IsXml;
end;

function TXmlEditFrame.CanCopy: Boolean;
begin
  Result := IsXml and FXmlEditor.CanCopy;
end;

function TXmlEditFrame.CanCut: Boolean;
begin
  Result := IsXml and FXmlEditor.CanCut;
end;

function TXmlEditFrame.CanDelete: Boolean;
begin
  Result := IsXml and FXmlEditor.CanDelete;
end;

function TXmlEditFrame.CanPaste: Boolean;
begin
  Result := IsXml and FXmlEditor.CanPaste;
end;

function TXmlEditFrame.CanPrint: Boolean;
begin
  Result := IsXml;
end;

function TXmlEditFrame.CanRedo: Boolean;
begin
  Result := IsXml and FXmlEditor.CanRedo;
end;

function TXmlEditFrame.CanReplace: Boolean;
begin
  Result := IsXml and FXmlEditor.CanReplace;
end;

function TXmlEditFrame.CanSave: Boolean;
var
  Path: TFilename;
  Authority, Query, Fragment: WideString; // dummies
begin
  UriWideStrToFilename(DocumentUri, Path, Authority, Query, Fragment);
  Result := IsXml and (Modified or (Path = ''));
end;

function TXmlEditFrame.CanSaveAs: Boolean;
begin
  Result := IsXml;
end;

function TXmlEditFrame.CanSearch: Boolean;
begin
  Result := IsXml;
end;

function TXmlEditFrame.CanSelectAll: Boolean;
begin
  Result := IsXml;
end;

function TXmlEditFrame.CanUndo: Boolean;
begin
  Result := IsXml and FXmlEditor.CanUndo;
end;

function TXmlEditFrame.CaretPosStr: string;
begin
  Result := FXmlEditor.CaretPosStr;
end;

function TXmlEditFrame.CheckValidity: Boolean;
begin
  UpdateXmlDocManager(False);
  Result := XmlDocManager.CheckValidity;
  UpdateDisplays(Modified);
  FocusEditor;
end;

function TXmlEditFrame.CheckWellformedness: Boolean;
begin
  UpdateXmlDocManager(False); // xxx False?
  Result := XmlDocManager.IsWellformed;
end;

procedure TXmlEditFrame.CopyToClipboard;
begin
  FXmlEditor.CopyToClipboard;
end;

constructor TXmlEditFrame.Create(AOwner: TComponent; const ASourceCode: TBytes;
  const ADocumentUri: WideString; const AModified: Boolean);
begin
  inherited Create(AOwner);
  FXmlEditor := XmlWinEdit;
  DocumentUri := ADocumentUri;
  LoadFromBytes(ASourceCode, AModified);
end;

procedure TXmlEditFrame.CutToClipboard;
begin
  FXmlEditor.CutToClipboard;
end;

procedure TXmlEditFrame.DeleteSelection;
begin
  FXmlEditor.DeleteSelection;
end;

destructor TXmlEditFrame.Destroy;
begin
  FXmlEditor := nil;
  inherited;
end;

procedure TXmlEditFrame.DoSetupHexEditorRequest;
begin
  if Assigned(FOnSetupHexEditorRequest) then FOnSetupHexEditorRequest(Self);
end;

function TXmlEditFrame.FocusEditor: Boolean;
begin
  if FXmlEditor.CanFocus then begin
    FXmlEditor.SetFocus;
    Result := True;
  end else
    Result := False;
end;

function TXmlEditFrame.GetDocumentUri: WideString;
begin
  Result := XmlDocManager.DocumentUri;
end;

function TXmlEditFrame.GetEditorFont: TFont;
begin
  Result := XmlWinEdit.EditorFont;
end;

function TXmlEditFrame.GetEditorState: string;
begin
  Result := FXmlEditor.GetEditorState;
end;

function TXmlEditFrame.GetErrorAutoStrings: TDomErrorAutoStrings;
begin
  Result := XmlDocManager.ErrorAutoStrings;
end;

function TXmlEditFrame.GetIsWellformed: Boolean;
begin
  Result := XmlDocManager.IsWellformed;
end;

function TXmlEditFrame.GetIsXml: Boolean;
begin
  Result := XmlWinEdit.Visible;
end;

function TXmlEditFrame.GetMimeType: string;
begin
  Result := XmlDocManager.MimeType;
end;

function TXmlEditFrame.GetModified: Boolean;
begin
  Result := FXmlEditor.Modified;
end;

function TXmlEditFrame.GetResourceResolver: TCustomResourceResolver;
begin
  Result := XmlDocManager.ResourceResolver;
end;

function TXmlEditFrame.GetXmlHighlighter: TSynCustomHighlighter;
begin
  Result := XmlWinEdit.SynEdit.Highlighter;
end;

function TXmlEditFrame.HasErrorStrings: Boolean;
begin
  Result := XmlDocManager.HasErrorStrings;
end;

procedure TXmlEditFrame.HideEditor;
begin
  RawDataMessagePanel.Show;
  XmlWinEdit.Hide;
end;

function TXmlEditFrame.IsEmpty: Boolean;
begin
  Result := FXmlEditor.IsEmpty;
end;

procedure TXmlEditFrame.LaunchHexEditorBtnClick(Sender: TObject);
var
  DocumentPath: TFilename;
  Authority, Query, Fragment: WideString; // dummies
begin
  UriWideStrToFilename(DocumentUri, DocumentPath, Authority, Query, Fragment);
  if not FileExists(ExternalHexEditor) then begin
    if MessageDlg(SHexEditorNotFound, mtError, [mbYes, mbNo], 0, mbYes) = mrYes then
      DoSetupHexEditorRequest;
  end else
  if ShellExecute(Application.Handle, 'open', PChar('"' + ExternalHexEditor + '"'),
    PChar('"' + DocumentPath + '"'), nil, SW_NORMAL) < 32 then
    if MessageDlg(SHexEditorLaunchFailed, mtError, [mbYes, mbNo], 0, mbYes) = mrYes then
      DoSetupHexEditorRequest;
end;

procedure TXmlEditFrame.LaunchPreferencesBtnClick(Sender: TObject);
begin
  DoSetupHexEditorRequest;
end;

procedure TXmlEditFrame.LoadFromBytes(const Data: TBytes;
                                      const AModified: Boolean);
begin
  XmlDocManager.SetRawData(Data, 'application/xml', '', False);
  UpdateDisplays(AModified);
end;

procedure TXmlEditFrame.PasteFromClipboard;
begin
  FXmlEditor.PasteFromClipboard;
end;

procedure TXmlEditFrame.Print(const ASynEditPrint: TSynEditPrint);
begin
  XmlWinEdit.SynEditPrint := ASynEditPrint; // Remark: Setting FXmlWinEditor.SynEditPrint automatically sets ASynEditPrint.SynEdit.
  ASynEditPrint.Title := DocumentUri;
  FXmlEditor.Print;
end;

procedure TXmlEditFrame.PrintPreview(const ASynEditPrint: TSynEditPrint;
  const PrinterSetupAction: TAction);
begin
  XmlWinEdit.SynEditPrint := ASynEditPrint; // Remark: Setting FXmlWinEditor.SynEditPrint automatically sets ASynEditPrint.SynEdit.
  ASynEditPrint.Title := DocumentUri;
  FXmlEditor.PrintPreview(PrinterSetupAction);
end;

procedure TXmlEditFrame.Redo;
begin
  FXmlEditor.Redo;
end;

function TXmlEditFrame.Save: Boolean;
var
  Path: TFilename;
  Authority, Query, Fragment: WideString; // dummies
  Data: TBytes;
  OutputEncoding: string;
begin
  { TODO -cTo Do : Support more output encoding schemes! }
  OutputEncoding := 'UTF-8';
  UriWideStrToFilename(DocumentUri, Path, Authority, Query, Fragment);
  try
    UpdateXmlDocManager(True);

    if XmlDocManager.MimeType = 'application/xml' then begin
      try
        Data := XmlDocManager.SerializeData(OutputEncoding, toEnforceXmlDecl, [bomUTF16, bomUCS2]);
      except
        Data := XmlDocManager.RawData;
        MessageBeep(MB_ICONEXCLAMATION);
        ShowMessage(Format(SCannotConvert, [OutputEncoding]));
      end;
    end else if XmlDocManager.MimeType = 'application/octet-stream' then begin
      Data := XmlDocManager.RawData;
    end else
      raise EFOpenError.CreateFmt(SUnknownMimeType, [XmlDocManager.MimeType]);

    SaveBytes(Data, Path);
    Modified := False;
    Result := True;
  except
    Application.HandleException(Self);
    Result := False;
  end;
end;

procedure TXmlEditFrame.SearchReplace(const ASearch, AReplace: WideString;
  AOptions: TSynSearchOptions);
begin
  FXmlEditor.SearchReplace(ASearch, AReplace, AOptions);
end;

procedure TXmlEditFrame.SelectAll;
begin
  FXmlEditor.SelectAll;
end;

function TXmlEditFrame.SetCaret(const Location: IDomLocator): Boolean;
var
  Line, Column: Int64;
begin
  with Location do
    if Uri = DocumentUri then begin
      if Assigned(RelatedNode) then begin
        GetNodeStartPosition(RelatedNode, Line, Column);
        Result := FXmlEditor.SetCaret(StartByteNumber, Line, Column);
      end else
        Result := FXmlEditor.SetCaret(StartByteNumber, StartLineNumber, StartColumnNumber);
    end else
      Result := False;
end;

procedure TXmlEditFrame.SetDocumentUri(const AUri: WideString);
begin
  XmlDocManager.DocumentUri := AUri;
end;

procedure TXmlEditFrame.SetEditorFont(const Value: TFont);
begin
  XmlWinEdit.EditorFont := Value;
end;

procedure TXmlEditFrame.SetExternalHexEditor(const Value: TFilename);
var
  HexEditorSpecified: Boolean;
begin
  FExternalHexEditor := Value;
  HexEditorSpecified := Trim(ExternalHexEditor) <> '';
  LaunchHexEditorPanel.Visible := HexEditorSpecified;
  ConfigureHexEditorPanel.Visible := not HexEditorSpecified;
end;

procedure TXmlEditFrame.SetModified(const Value: Boolean);
begin
  FXmlEditor.Modified := Value;
end;

procedure TXmlEditFrame.SetResourceResolver(
  const Value: TCustomResourceResolver);
begin
  XmlDocManager.ResourceResolver := Value;
end;

procedure TXmlEditFrame.SetTextWin(AText: TBytes;
                                   AModified: Boolean);
begin
  XmlWinEdit.Data := AText;
  XmlWinEdit.Modified := AModified;
end;

procedure TXmlEditFrame.SetXmlHighlighter(const Value: TSynCustomHighlighter);
begin
  XmlWinEdit.SynEdit.Highlighter := Value;
end;

procedure TXmlEditFrame.ShowEditor;
begin
  XmlWinEdit.Show;
  RawDataMessagePanel.Hide;
end;

function TXmlEditFrame.StartReplace(var Params: TXmlSearchParams): Boolean;
begin
  Result := FXmlEditor.StartReplace(Params);
end;

function TXmlEditFrame.StartSearch(var Params: TXmlSearchParams): Boolean;
begin
  Result := FXmlEditor.StartSearch(Params);
end;

procedure TXmlEditFrame.Undo;
begin
  FXmlEditor.Undo;
end;

procedure TXmlEditFrame.UpdateDisplays(const AModified: Boolean);
// Updates the displays with the contents of the XML Document Manager.
begin
  if XmlDocManager.MimeType = 'application/xml' then
  begin
    try
      SetTextWin(XmlDocManager.GetUnicodeData, AModified);
      ShowEditor;
    except
      HideEditor;
    end;
  end else
    HideEditor;
end;

procedure TXmlEditFrame.UpdateXmlDocManager(const IncDecl: Boolean);
begin
  XmlDocManager.SetRawData(FXmlEditor.Data, 'application/xml', 'UTF-16LE', IncDecl);
end;

end.
