// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// "http://www.mozilla.org/MPL/"
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "uXmlEditTypes.pas".
//
// The Initial Developer of the Original Code is Dieter Khler (Heidelberg,
// Germany, "http://www.philo.de/").
// Portions created by the Initial Developer are Copyright (C) 2004-2008
// Dieter Khler. All Rights Reserved.
//
// Alternatively, the contents of this file may be used under the terms of
// the GNU General Public License Version 2 or later (the "GPL"), in which
// case the provisions of the GPL are applicable instead of those above.
// If you wish to allow use of your version of this file only under the
// terms of the GPL, and not to allow others to use your version of this
// file under the terms of the MPL, indicate your decision by deleting the
// provisions above and replace them with the notice and other provisions
// required by the GPL. If you do not delete the provisions above, a
// recipient may use your version of this file under the terms of any one
// of the MPL or the GPL.
//
unit uXmlEditTypes;

interface

uses
  SynEditTypes, ActnList, SysUtils;

type
  TXmlSearchParams = record
    SearchBackwards: Boolean;
    SearchCaseSensitive: Boolean;
    SearchFromCaret: Boolean;
    SearchFromCursor: Boolean;
    SearchSelectionOnly: Boolean;
    SearchTextAtCaret: Boolean;
    SearchWholeWords: Boolean;

    SearchText: WideString;
    SearchTextHistory: WideString;
    ReplaceText: WideString;
    ReplaceTextHistory: WideString;
  end;

  IXmlEditCtrl = interface
    function CanCopy: Boolean; stdcall;
    function CanCut: Boolean; stdcall;
    function CanFocus: Boolean; stdcall;
    function CanDelete: Boolean; stdcall;
    function CanPaste: Boolean; stdcall;
    function CanRedo: Boolean; stdcall;
    function CanReplace: Boolean; stdcall;
    function CanUndo: Boolean; stdcall;
    function CaretPosStr: string; stdcall;
    procedure CopyToClipboard; stdcall;
    procedure CutToClipboard; stdcall;
    procedure DeleteSelection; stdcall;
    function GetEditorState: string; stdcall;
    function GetModified: Boolean; stdcall;
    function GetData: TBytes; stdcall;
    function IsEmpty: Boolean; stdcall;
    procedure PasteFromClipboard; stdcall;
    procedure Print; stdcall;
    procedure PrintPreview(const PrinterSetupAction: TAction); stdcall;
    procedure Redo; stdcall;
    procedure SearchReplace(const ASearch, AReplace: WideString;
      AOptions: TSynSearchOptions); stdcall;             // xxx remove TSynSearchOptions?
    procedure SelectAll; stdcall;
    function SetCaret(const ByteOffset, Line, Column: Integer): Boolean; stdcall;
    procedure SetFocus; stdcall;
    procedure SetModified(const Value: Boolean); stdcall;
    function StartReplace(var Params: TXmlSearchParams): Boolean; stdcall;
    function StartSearch(var Params: TXmlSearchParams): Boolean; stdcall;
    procedure Undo; stdcall;

    property Modified: Boolean read GetModified write SetModified;
    property Data: TBytes read GetData;
  end;

implementation

end.
